import { Task } from "..";
import { Component } from "../component";
import { GitIdentity } from "../github";
import { Job, JobStep, Tools } from "../github/workflows-model";
import { Project } from "../project";
/**
 * @experimental
 */
export interface BuildWorkflowOptions {
    /**
     * (experimental) The task to execute in order to build the project.
     *
     * @experimental
     */
    readonly buildTask: Task;
    /**
     * (experimental) A name of a directory that includes build artifacts.
     *
     * @experimental
     */
    readonly artifactsDirectory: string;
    /**
     * (experimental) The container image to use for builds.
     *
     * @default - the default workflow container
     * @experimental
     */
    readonly containerImage?: string;
    /**
     * (experimental) Automatically update files modified during builds to pull-request branches.
     *
     * This means that any files synthesized by projen or e.g. test snapshots will
     * always be up-to-date before a PR is merged.
     *
     * Implies that PR builds do not have anti-tamper checks.
     *
     * This is enabled by default only if `githubTokenSecret` is set. Otherwise it
     * is disabled, which implies that file changes that happen during build will
     * not be pushed back to the branch.
     *
     * @default true
     * @experimental
     */
    readonly mutableBuild?: boolean;
    /**
     * (experimental) Steps to execute before the build.
     *
     * @default []
     * @experimental
     */
    readonly preBuildSteps?: JobStep[];
    /**
     * (experimental) Steps to execute after build.
     *
     * @default []
     * @experimental
     */
    readonly postBuildSteps?: JobStep[];
    /**
     * (experimental) Git identity to use for the workflow.
     *
     * @default - default identity
     * @experimental
     */
    readonly gitIdentity?: GitIdentity;
    /**
     * (experimental) Build environment variables.
     *
     * @default {}
     * @experimental
     */
    readonly env?: {
        [key: string]: string;
    };
    /**
     * (experimental) Github Runner selection labels.
     *
     * @default ["ubuntu-latest"]
     * @experimental
     */
    readonly runsOn?: string[];
}
/**
 * @experimental
 */
export declare class BuildWorkflow extends Component {
    private readonly postBuildSteps;
    private readonly preBuildSteps;
    private readonly gitIdentity;
    private readonly buildTask;
    private readonly github;
    private readonly workflow;
    private readonly artifactsDirectory;
    private readonly defaultRunners;
    private readonly _postBuildJobs;
    /**
     * @experimental
     */
    constructor(project: Project, options: BuildWorkflowOptions);
    private addBuildJob;
    /**
     * (experimental) Returns a list of job IDs that are part of the build.
     *
     * @experimental
     */
    get buildJobIds(): string[];
    /**
     * (experimental) Adds steps that are executed after the build.
     *
     * @param steps The job steps.
     * @experimental
     */
    addPostBuildSteps(...steps: JobStep[]): void;
    /**
     * (experimental) Adds another job to the build workflow which is executed after the build job succeeded.
     *
     * Jobs are executed _only_ if the build did NOT self mutate. If the build
     * self-mutate, the branch will either be updated or the build will fail (in
     * forks), so there is no point in executing the post-build job.
     *
     * @param id The id of the new job.
     * @param job The job specification.
     * @experimental
     */
    addPostBuildJob(id: string, job: Job): void;
    /**
     * (experimental) Run a task as a job within the build workflow which is executed after the build job succeeded.
     *
     * The job will have access to build artifacts and will install project
     * dependencies in order to be able to run any commands used in the tasks.
     *
     * Jobs are executed _only_ if the build did NOT self mutate. If the build
     * self-mutate, the branch will either be updated or the build will fail (in
     * forks), so there is no point in executing the post-build job.
     *
     * @param options Specify tools and other options.
     * @experimental
     */
    addPostBuildJobTask(task: Task, options: AddPostBuildJobTaskOptions): void;
    /**
     * (experimental) Run a sequence of commands as a job within the build workflow which is executed after the build job succeeded.
     *
     * Jobs are executed _only_ if the build did NOT self mutate. If the build
     * self-mutate, the branch will either be updated or the build will fail (in
     * forks), so there is no point in executing the post-build job.
     *
     * @param options Specify tools and other options.
     * @experimental
     */
    addPostBuildJobCommands(id: string, commands: string[], options?: AddPostBuildJobCommandsOptions): void;
    private addSelfMutationJob;
    /**
     * Called (lazily) during synth to render the build job steps.
     */
    private renderBuildSteps;
}
/**
 * (experimental) Options for `BuildWorkflow.addPostBuildJobTask`.
 *
 * @experimental
 */
export interface AddPostBuildJobTaskOptions {
    /**
     * (experimental) Tools that should be installed before the task is run.
     *
     * @experimental
     */
    readonly tools?: Tools;
    /**
     * (experimental) Github Runner selection labels.
     *
     * @default ["ubuntu-latest"]
     * @experimental
     */
    readonly runsOn?: string[];
}
/**
 * (experimental) Options for `BuildWorkflow.addPostBuildJobCommands`.
 *
 * @experimental
 */
export interface AddPostBuildJobCommandsOptions {
    /**
     * (experimental) Tools that should be installed before the commands are run.
     *
     * @experimental
     */
    readonly tools?: Tools;
    /**
     * (experimental) Check out the repository at the pull request branch before commands are run.
     *
     * @default false
     * @experimental
     */
    readonly checkoutRepo?: boolean;
    /**
     * (experimental) Install project dependencies before running commands. `checkoutRepo` must also be set to true.
     *
     * Currently only supported for `NodeProject`.
     *
     * @default false
     * @experimental
     */
    readonly installDeps?: boolean;
    /**
     * (experimental) Github Runner selection labels.
     *
     * @default ["ubuntu-latest"]
     * @experimental
     */
    readonly runsOn?: string[];
}
