/**
 * (experimental) Cache Definition.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#cache
 * @experimental
 */
export interface Cache {
    /**
     * (experimental) Defines when to save the cache, based on the status of the job (Default: Job Success).
     *
     * @experimental
     */
    readonly when?: CacheWhen;
}
/**
 * (experimental) Configure when artifacts are uploaded depended on job status.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#cachewhen
 * @experimental
 */
export declare enum CacheWhen {
    /**
     * (experimental) Upload artifacts regardless of job status.
     *
     * @experimental
     */
    ALWAYS = "always",
    /**
     * (experimental) Upload artifacts only when the job fails.
     *
     * @experimental
     */
    ON_FAILURE = "on_failure",
    /**
     * (experimental) Upload artifacts only when the job succeeds (this is the default).
     *
     * @experimental
     */
    ON_SUCCESS = "on_success"
}
/**
 * (experimental) Default settings for the CI Configuration.
 *
 * Jobs that do not define one or more of the listed keywords use the value defined in the default section.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#default
 * @experimental
 */
export interface Default {
    /**
     * @experimental
     */
    readonly afterScript?: string[];
    /**
     * @experimental
     */
    readonly artifacts?: Artifacts;
    /**
     * @experimental
     */
    readonly beforeScript?: string[];
    /**
     * @experimental
     */
    readonly cache?: Cache;
    /**
     * @experimental
     */
    readonly image?: Image;
    /**
     * @experimental
     */
    readonly interruptible?: boolean;
    /**
     * @experimental
     */
    readonly retry?: Retry;
    /**
     * @experimental
     */
    readonly services?: Service[];
    /**
     * @experimental
     */
    readonly tags?: string[];
    /**
     * @experimental
     */
    readonly timeout?: string;
}
/**
 * (experimental) Used to specify a list of files and directories that should be attached to the job if it succeeds.
 *
 * Artifacts are sent to Gitlab where they can be downloaded.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#artifacts
 * @experimental
 */
export interface Artifacts {
    /**
     * (experimental) A list of paths to files/folders that should be excluded in the artifact.
     *
     * @experimental
     */
    readonly exclude?: string[];
    /**
     * (experimental) How long artifacts should be kept.
     *
     * They are saved 30 days by default. Artifacts that have expired are removed periodically via cron job. Supports a wide variety of formats, e.g. '1 week', '3 mins 4 sec', '2 hrs 20 min', '2h20min', '6 mos 1 day', '47 yrs 6 mos and 4d', '3 weeks and 2 days'.
     *
     * @experimental
     */
    readonly expireIn?: string;
    /**
     * (experimental) Can be used to expose job artifacts in the merge request UI.
     *
     * GitLab will add a link <expose_as> to the relevant merge request that points to the artifact.
     *
     * @experimental
     */
    readonly exposeAs?: string;
    /**
     * (experimental) Name for the archive created on job success.
     *
     * Can use variables in the name, e.g. '$CI_JOB_NAME'
     *
     * @experimental
     */
    readonly name?: string;
    /**
     * (experimental) A list of paths to files/folders that should be included in the artifact.
     *
     * @experimental
     */
    readonly paths?: string[];
    /**
     * (experimental) Reports will be uploaded as artifacts, and often displayed in the Gitlab UI, such as in Merge Requests.
     *
     * @experimental
     */
    readonly reports?: Reports;
    /**
     * (experimental) Whether to add all untracked files (along with 'artifacts.paths') to the artifact.
     *
     * @experimental
     */
    readonly untracked?: boolean;
    /**
     * (experimental) Configure when artifacts are uploaded depended on job status.
     *
     * @experimental
     */
    readonly when?: CacheWhen;
}
/**
 * (experimental) Reports will be uploaded as artifacts, and often displayed in the Gitlab UI, such as in Merge Requests.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#artifactsreports
 * @experimental
 */
export interface Reports {
    /**
     * (experimental) Path for file(s) that should be parsed as Cobertura XML coverage report.
     *
     * @experimental
     */
    readonly cobertura?: string[];
    /**
     * (experimental) Path to file or list of files with code quality report(s) (such as Code Climate).
     *
     * @experimental
     */
    readonly codequality?: string[];
    /**
     * (experimental) Path to file or list of files with Container scanning vulnerabilities report(s).
     *
     * @experimental
     */
    readonly containerScanning?: string[];
    /**
     * (experimental) Path to file or list of files with DAST vulnerabilities report(s).
     *
     * @experimental
     */
    readonly dast?: string[];
    /**
     * (experimental) Path to file or list of files with Dependency scanning vulnerabilities report(s).
     *
     * @experimental
     */
    readonly dependencyScanning?: string[];
    /**
     * (experimental) Path to file or list of files containing runtime-created variables for this job.
     *
     * @experimental
     */
    readonly dotenv?: string[];
    /**
     * (experimental) Path for file(s) that should be parsed as JUnit XML result.
     *
     * @experimental
     */
    readonly junit?: string[];
    /**
     * (experimental) Deprecated in 12.8: Path to file or list of files with license report(s).
     *
     * @experimental
     */
    readonly licenseManagement?: string[];
    /**
     * (experimental) Path to file or list of files with license report(s).
     *
     * @experimental
     */
    readonly licenseScanning?: string[];
    /**
     * (experimental) Path to file or list of files containing code intelligence (Language Server Index Format).
     *
     * @experimental
     */
    readonly lsif?: string[];
    /**
     * (experimental) Path to file or list of files with custom metrics report(s).
     *
     * @experimental
     */
    readonly metrics?: string[];
    /**
     * (experimental) Path to file or list of files with performance metrics report(s).
     *
     * @experimental
     */
    readonly performance?: string[];
    /**
     * (experimental) Path to file or list of files with requirements report(s).
     *
     * @experimental
     */
    readonly requirements?: string[];
    /**
     * (experimental) Path to file or list of files with SAST vulnerabilities report(s).
     *
     * @experimental
     */
    readonly sast?: string[];
    /**
     * (experimental) Path to file or list of files with secret detection report(s).
     *
     * @experimental
     */
    readonly secretDetection?: string[];
    /**
     * (experimental) Path to file or list of files with terraform plan(s).
     *
     * @experimental
     */
    readonly terraform?: string[];
}
/**
 * (experimental) Specifies the docker image to use for the job or globally for all jobs.
 *
 * Job configuration
 * takes precedence over global setting. Requires a certain kind of Gitlab runner executor.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#image
 * @experimental
 */
export interface Image {
    /**
     * (experimental) Command or script that should be executed as the container's entrypoint.
     *
     * It will be translated to Docker's --entrypoint option while creating the container. The syntax is similar to Dockerfile's ENTRYPOINT directive, where each shell token is a separate string in the array.
     *
     * @experimental
     */
    readonly entrypoint?: any[];
    /**
     * (experimental) Full name of the image that should be used.
     *
     * It should contain the Registry part if needed.
     *
     * @experimental
     */
    readonly name: string;
}
/**
 * (experimental) How many times a job is retried if it fails.
 *
 * If not defined, defaults to 0 and jobs do not retry.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#retry
 * @experimental
 */
export interface Retry {
    /**
     * (experimental) 0 (default), 1, or 2.
     *
     * @experimental
     */
    readonly max?: number;
    /**
     * (experimental) Either a single or array of error types to trigger job retry.
     *
     * @experimental
     */
    readonly when?: any;
}
/**
 * (experimental) Used to specify an additional Docker image to run scripts in.
 *
 * The service image is linked to the image specified in the @Default image keyword.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#services
 * @experimental
 */
export interface Service {
    /**
     * (experimental) Additional alias that can be used to access the service from the job's container.
     *
     * Read Accessing the services for more information.
     *
     * @experimental
     */
    readonly alias?: string;
    /**
     * (experimental) Command or script that should be used as the container's command.
     *
     * It will be translated to arguments passed to Docker after the image's name. The syntax is similar to Dockerfile's CMD directive, where each shell token is a separate string in the array.
     *
     * @experimental
     */
    readonly command?: string[];
    /**
     * (experimental) Command or script that should be executed as the container's entrypoint.
     *
     * It will be translated to Docker's --entrypoint option while creating the container. The syntax is similar to Dockerfile's ENTRYPOINT directive, where each shell token is a separate string in the array.
     *
     * @experimental
     */
    readonly entrypoint?: string[];
    /**
     * (experimental) Full name of the image that should be used.
     *
     * It should contain the Registry part if needed.
     *
     * @experimental
     */
    readonly name: string;
}
/**
 * (experimental) An included YAML file.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#include
 * @experimental
 */
export interface Include {
    /**
     * (experimental) Relative path from local repository root (`/`) to the `yaml`/`yml` file template.
     *
     * The file must be on the same branch, and does not work across git submodules.
     *
     * @experimental
     */
    readonly local?: string;
    /**
     * (experimental) Rules allows for an array of individual rule objects to be evaluated in order, until one matches and dynamically provides attributes to the job.
     *
     * @experimental
     */
    readonly rules?: IncludeRule[];
    /**
     * (experimental) Files from another private project on the same GitLab instance.
     *
     * You can use `file` in combination with `project` only.
     *
     * @experimental
     */
    readonly file?: string[];
    /**
     * (experimental) Path to the project, e.g. `group/project`, or `group/sub-group/project`.
     *
     * @experimental
     */
    readonly project?: string;
    /**
     * (experimental) Branch/Tag/Commit-hash for the target project.
     *
     * @experimental
     */
    readonly ref?: string;
    /**
     * (experimental) Use a `.gitlab-ci.yml` template as a base, e.g. `Nodejs.gitlab-ci.yml`.
     *
     * @experimental
     */
    readonly template?: string;
    /**
     * (experimental) URL to a `yaml`/`yml` template file using HTTP/HTTPS.
     *
     * @experimental
     */
    readonly remote?: string;
}
/**
 * (experimental) Rules allows for an array of individual rule objects to be evaluated in order, until one matches and dynamically provides attributes to the job.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/includes.html#use-rules-with-include
 * @experimental
 */
export interface IncludeRule {
    /**
     * @experimental
     */
    readonly allowFailure?: boolean | AllowFailure;
    /**
     * @experimental
     */
    readonly changes?: string[];
    /**
     * @experimental
     */
    readonly exists?: string[];
    /**
     * @experimental
     */
    readonly if?: string;
    /**
     * @experimental
     */
    readonly startIn?: string;
    /**
     * @experimental
     */
    readonly variables?: Record<string, number | string>;
    /**
     * @experimental
     */
    readonly when?: JobWhen;
}
/**
 * (experimental) Exit code that are not considered failure.
 *
 * The job fails for any other exit code.
 * You can list which exit codes are not considered failures. The job fails for any other
 * exit code.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#allow_failure
 * @experimental
 */
export interface AllowFailure {
    /**
     * @experimental
     */
    readonly exitCodes: number[] | number;
}
/**
 * (experimental) Describes the conditions for when to run the job.
 *
 * Defaults to 'on_success'.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#when
 * @experimental
 */
export declare enum JobWhen {
    /**
     * @experimental
     */
    ALWAYS = "always",
    /**
     * @experimental
     */
    DELAYED = "delayed",
    /**
     * @experimental
     */
    MANUAL = "manual",
    /**
     * @experimental
     */
    NEVER = "never",
    /**
     * @experimental
     */
    ON_FAILURE = "on_failure",
    /**
     * @experimental
     */
    ON_SUCCESS = "on_success"
}
/**
 * (experimental) Jobs are the most fundamental element of a .gitlab-ci.yml file.
 *
 * @see https://docs.gitlab.com/ee/ci/jobs/
 * @experimental
 */
export interface Job {
    /**
     * @experimental
     */
    readonly afterScript?: string[];
    /**
     * (experimental) Whether to allow the pipeline to continue running on job failure (Default: false).
     *
     * @experimental
     */
    readonly allowFailure?: boolean | AllowFailure;
    /**
     * @experimental
     */
    readonly artifacts?: Artifacts;
    /**
     * @experimental
     */
    readonly beforeScript?: string[];
    /**
     * @experimental
     */
    readonly cache?: Cache;
    /**
     * (experimental) Must be a regular expression, optionally but recommended to be quoted, and must be surrounded with '/'.
     *
     * Example: '/Code coverage: \d+\.\d+/'
     *
     * @experimental
     */
    readonly coverage?: string;
    /**
     * (experimental) Specify a list of job names from earlier stages from which artifacts should be loaded.
     *
     * By default, all previous artifacts are passed. Use an empty array to skip downloading artifacts.
     *
     * @experimental
     */
    readonly dependencies?: string[];
    /**
     * (experimental) Used to associate environment metadata with a deploy.
     *
     * Environment can have a name and URL attached to it, and will be displayed under /environments under the project.
     *
     * @experimental
     */
    readonly environment?: Environment | string;
    /**
     * (experimental) Job will run *except* for when these filtering options match.
     *
     * @experimental
     */
    readonly except?: string[] | Filter;
    /**
     * (experimental) The name of one or more jobs to inherit configuration from.
     *
     * @experimental
     */
    readonly extends?: string[];
    /**
     * @experimental
     */
    readonly image?: Image;
    /**
     * (experimental) Controls inheritance of globally-defined defaults and variables.
     *
     * Boolean values control inheritance of all default: or variables: keywords. To inherit only a subset of default: or variables: keywords, specify what you wish to inherit. Anything not listed is not inherited.
     *
     * @experimental
     */
    readonly inherit?: Inherit;
    /**
     * @experimental
     */
    readonly interruptible?: boolean;
    /**
     * (experimental) The list of jobs in previous stages whose sole completion is needed to start the current job.
     *
     * @experimental
     */
    readonly needs?: Array<Need | string>;
    /**
     * (experimental) Job will run *only* when these filtering options match.
     *
     * @experimental
     */
    readonly only?: string[] | Filter;
    /**
     * (experimental) Parallel will split up a single job into several, and provide `CI_NODE_INDEX` and `CI_NODE_TOTAL` environment variables for the running jobs.
     *
     * @experimental
     */
    readonly parallel?: Parallel | number;
    /**
     * (experimental) Indicates that the job creates a Release.
     *
     * @experimental
     */
    readonly release?: Release;
    /**
     * (experimental) Limit job concurrency.
     *
     * Can be used to ensure that the Runner will not run certain jobs simultaneously.
     *
     * @experimental
     */
    readonly resourceGroup?: string;
    /**
     * @experimental
     */
    readonly retry?: Retry;
    /**
     * (experimental) Rules allows for an array of individual rule objects to be evaluated in order, until one matches and dynamically provides attributes to the job.
     *
     * @experimental
     */
    readonly rules?: IncludeRule[];
    /**
     * (experimental) Shell scripts executed by the Runner.
     *
     * The only required property of jobs. Be careful with special characters (e.g. `:`, `{`, `}`, `&`) and use single or double quotes to avoid issues.
     *
     * @experimental
     */
    readonly script?: string[];
    /**
     * (experimental) CI/CD secrets.
     *
     * @experimental
     */
    readonly secrets?: Record<string, Record<string, Secret>>;
    /**
     * @experimental
     */
    readonly services?: Service[];
    /**
     * (experimental) Define what stage the job will run in.
     *
     * @experimental
     */
    readonly stage?: string;
    /**
     * @experimental
     */
    readonly startIn?: string;
    /**
     * @experimental
     */
    readonly tags?: string[];
    /**
     * @experimental
     */
    readonly timeout?: string;
    /**
     * (experimental) Trigger allows you to define downstream pipeline trigger.
     *
     * When a job created from trigger definition is started by GitLab, a downstream pipeline gets created. Read more: https://docs.gitlab.com/ee/ci/yaml/README.html#trigger
     *
     * @experimental
     */
    readonly trigger?: Trigger | string;
    /**
     * (experimental) Configurable values that are passed to the Job.
     *
     * @experimental
     */
    readonly variables?: Record<string, number | string>;
    /**
     * (experimental) Describes the conditions for when to run the job.
     *
     * Defaults to 'on_success'.
     *
     * @experimental
     */
    readonly when?: JobWhen;
}
/**
 * (experimental) The environment that a job deploys to.
 *
 * @experimental
 */
export interface Environment {
    /**
     * (experimental) Specifies what this job will do.
     *
     * 'start' (default) indicates the job will start the deployment. 'prepare' indicates this will not affect the deployment. 'stop' indicates this will stop the deployment.
     *
     * @experimental
     */
    readonly action?: Action;
    /**
     * (experimental) The amount of time it should take before Gitlab will automatically stop the environment.
     *
     * Supports a wide variety of formats, e.g. '1 week', '3 mins 4 sec', '2 hrs 20 min', '2h20min', '6 mos 1 day', '47 yrs 6 mos and 4d', '3 weeks and 2 days'.
     *
     * @experimental
     */
    readonly autoStopIn?: string;
    /**
     * (experimental) Explicitly specifies the tier of the deployment environment if non-standard environment name is used.
     *
     * @experimental
     */
    readonly deploymentTier?: DeploymentTier;
    /**
     * (experimental) Used to configure the kubernetes deployment for this environment.
     *
     * This is currently not supported for kubernetes clusters that are managed by Gitlab.
     *
     * @experimental
     */
    readonly kubernetes?: KubernetesConfig;
    /**
     * (experimental) The name of the environment, e.g. 'qa', 'staging', 'production'.
     *
     * @experimental
     */
    readonly name: string;
    /**
     * (experimental) The name of a job to execute when the environment is about to be stopped.
     *
     * @experimental
     */
    readonly onStop?: string;
    /**
     * (experimental) When set, this will expose buttons in various places for the current environment in Gitlab, that will take you to the defined URL.
     *
     * @experimental
     */
    readonly url?: string;
}
/**
 * (experimental) Specifies what this job will do.
 *
 * 'start' (default) indicates the job will start the
 * deployment. 'prepare' indicates this will not affect the deployment. 'stop' indicates
 * this will stop the deployment.
 *
 * @experimental
 */
export declare enum Action {
    /**
     * @experimental
     */
    PREPARE = "prepare",
    /**
     * @experimental
     */
    START = "start",
    /**
     * @experimental
     */
    STOP = "stop"
}
/**
 * (experimental) Explicitly specifies the tier of the deployment environment if non-standard environment name is used.
 *
 * @experimental
 */
export declare enum DeploymentTier {
    /**
     * @experimental
     */
    DEVELOPMENT = "development",
    /**
     * @experimental
     */
    OTHER = "other",
    /**
     * @experimental
     */
    PRODUCTION = "production",
    /**
     * @experimental
     */
    STAGING = "staging",
    /**
     * @experimental
     */
    TESTING = "testing"
}
/**
 * (experimental) Used to configure the kubernetes deployment for this environment.
 *
 * This is currently not
 * supported for kubernetes clusters that are managed by Gitlab.
 *
 * @experimental
 */
export interface KubernetesConfig {
    /**
     * (experimental) The kubernetes namespace where this environment should be deployed to.
     *
     * @experimental
     */
    readonly namespace?: string;
}
/**
 * (experimental) Filtering options for when a job will run.
 *
 * @experimental
 */
export interface Filter {
    /**
     * (experimental) Filter job creation based on files that were modified in a git push.
     *
     * @experimental
     */
    readonly changes?: string[];
    /**
     * (experimental) Filter job based on if Kubernetes integration is active.
     *
     * @experimental
     */
    readonly kubernetes?: KubernetesEnum;
    /**
     * (experimental) Control when to add jobs to a pipeline based on branch names or pipeline types.
     *
     * @experimental
     */
    readonly refs?: string[];
    /**
     * (experimental) Filter job by checking comparing values of environment variables.
     *
     * Read more about variable expressions: https://docs.gitlab.com/ee/ci/variables/README.html#variables-expressions
     *
     * @experimental
     */
    readonly variables?: string[];
}
/**
 * Filter job based on if Kubernetes integration is active.
 */
export declare enum KubernetesEnum {
    /**
     * (experimental) Filter job based on if Kubernetes integration is active.
     *
     * @experimental
     */
    ACTIVE = "active"
}
/**
 * (experimental) Controls inheritance of globally-defined defaults and variables.
 *
 * Boolean values control
 * inheritance of all default: or variables: keywords. To inherit only a subset of default:
 * or variables: keywords, specify what you wish to inherit. Anything not listed is not
 * inherited.
 *
 * @experimental
 */
export interface Inherit {
    /**
     * (experimental) Whether to inherit all globally-defined defaults or not.
     *
     * Or subset of inherited defaults
     *
     * @experimental
     */
    readonly default?: DefaultElement[] | boolean;
    /**
     * (experimental) Whether to inherit all globally-defined variables or not.
     *
     * Or subset of inherited variables
     *
     * @experimental
     */
    readonly variables?: string[] | boolean;
}
/**
 * @experimental
 */
export declare enum DefaultElement {
    /**
     * @experimental
     */
    AFTER_SCRIPT = "after_script",
    /**
     * @experimental
     */
    ARTIFACTS = "artifacts",
    /**
     * @experimental
     */
    BEFORE_SCRIPT = "before_script",
    /**
     * @experimental
     */
    CACHE = "cache",
    /**
     * @experimental
     */
    IMAGE = "image",
    /**
     * @experimental
     */
    INTERRUPTIBLE = "interruptible",
    /**
     * @experimental
     */
    RETRY = "retry",
    /**
     * @experimental
     */
    SERVICES = "services",
    /**
     * @experimental
     */
    TAGS = "tags",
    /**
     * @experimental
     */
    TIMEOUT = "timeout"
}
/**
 * (experimental) A jobs in a previous stage whose sole completion is needed to start the current job.
 *
 * @experimental
 */
export interface Need {
    /**
     * @experimental
     */
    readonly artifacts?: boolean;
    /**
     * @experimental
     */
    readonly job: string;
    /**
     * @experimental
     */
    readonly optional?: boolean;
    /**
     * @experimental
     */
    readonly pipeline?: string;
    /**
     * @experimental
     */
    readonly project?: string;
    /**
     * @experimental
     */
    readonly ref?: string;
}
/**
 * (experimental) Used to run a job multiple times in parallel in a single pipeline.
 *
 * @experimental
 */
export interface Parallel {
    /**
     * (experimental) Defines different variables for jobs that are running in parallel.
     *
     * @experimental
     */
    readonly matrix: Record<string, any[]>[];
}
/**
 * (experimental) Indicates that the job creates a Release.
 *
 * @experimental
 */
export interface Release {
    /**
     * @experimental
     */
    readonly assets?: Assets;
    /**
     * (experimental) Specifies the longer description of the Release.
     *
     * @experimental
     */
    readonly description: string;
    /**
     * (experimental) The title of each milestone the release is associated with.
     *
     * @experimental
     */
    readonly milestones?: string[];
    /**
     * (experimental) The Release name.
     *
     * If omitted, it is populated with the value of release: tag_name.
     *
     * @experimental
     */
    readonly name?: string;
    /**
     * (experimental) If the release: tag_name doesn’t exist yet, the release is created from ref.
     *
     * ref can be a commit SHA, another tag name, or a branch name.
     *
     * @experimental
     */
    readonly ref?: string;
    /**
     * (experimental) The date and time when the release is ready.
     *
     * Defaults to the current date and time if not defined. Should be enclosed in quotes and expressed in ISO 8601 format.
     *
     * @experimental
     */
    readonly releasedAt?: string;
    /**
     * (experimental) The tag_name must be specified.
     *
     * It can refer to an existing Git tag or can be specified by the user.
     *
     * @experimental
     */
    readonly tagName: string;
}
/**
 * (experimental) Asset configuration for a release.
 *
 * @experimental
 */
export interface Assets {
    /**
     * (experimental) Include asset links in the release.
     *
     * @experimental
     */
    readonly links: Link[];
}
/**
 * (experimental) Link configuration for an asset.
 *
 * @experimental
 */
export interface Link {
    /**
     * (experimental) The redirect link to the url.
     *
     * @experimental
     */
    readonly filepath?: string;
    /**
     * (experimental) The content kind of what users can download via url.
     *
     * @experimental
     */
    readonly linkType?: LinkType;
    /**
     * (experimental) The name of the link.
     *
     * @experimental
     */
    readonly name: string;
    /**
     * (experimental) The URL to download a file.
     *
     * @experimental
     */
    readonly url: string;
}
/**
 * (experimental) The content kind of what users can download via url.
 *
 * @experimental
 */
export declare enum LinkType {
    /**
     * @experimental
     */
    IMAGE = "image",
    /**
     * @experimental
     */
    OTHER = "other",
    /**
     * @experimental
     */
    PACKAGE = "package",
    /**
     * @experimental
     */
    RUNBOOK = "runbook"
}
/**
 * (experimental) A CI/CD secret.
 *
 * @experimental
 */
export interface Secret {
    /**
     * @experimental
     */
    readonly vault: VaultConfig;
}
/**
 * (experimental) Specification for a secret provided by a HashiCorp Vault.
 *
 * @see https://www.vaultproject.io/
 * @experimental
 */
export interface VaultConfig {
    /**
     * @experimental
     */
    readonly engine: Engine;
    /**
     * @experimental
     */
    readonly field: string;
    /**
     * (experimental) Path to the secret.
     *
     * @experimental
     */
    readonly path: string;
}
/**
 * (experimental) The engine configuration for a secret.
 *
 * @experimental
 */
export interface Engine {
    /**
     * (experimental) Name of the secrets engine.
     *
     * @experimental
     */
    readonly name: string;
    /**
     * (experimental) Path to the secrets engine.
     *
     * @experimental
     */
    readonly path: string;
}
/**
 * (experimental) Trigger a multi-project or a child pipeline.
 *
 * Read more:
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/README.html#trigger-syntax-for-child-pipeline
 * @experimental
 */
export interface Trigger {
    /**
     * (experimental) The branch name that a downstream pipeline will use.
     *
     * @experimental
     */
    readonly branch?: string;
    /**
     * (experimental) Path to the project, e.g. `group/project`, or `group/sub-group/project`.
     *
     * @experimental
     */
    readonly project?: string;
    /**
     * (experimental) You can mirror the pipeline status from the triggered pipeline to the source bridge job by using strategy: depend.
     *
     * @experimental
     */
    readonly strategy?: Strategy;
    /**
     * (experimental) A list of local files or artifacts from other jobs to define the pipeline.
     *
     * @experimental
     */
    readonly include?: TriggerInclude[];
}
/**
 * (experimental) References a local file or an artifact from another job to define the pipeline configuration.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#triggerinclude
 * @experimental
 */
export interface TriggerInclude {
    /**
     * (experimental) Relative path from local repository root (`/`) to the local YAML file to define the pipeline configuration.
     *
     * @experimental
     */
    readonly local?: string;
    /**
     * (experimental) Name of the template YAML file to use in the pipeline configuration.
     *
     * @experimental
     */
    readonly template?: string;
    /**
     * (experimental) Relative path to the generated YAML file which is extracted from the artifacts and used as the configuration for triggering the child pipeline.
     *
     * @experimental
     */
    readonly artifact?: string;
    /**
     * (experimental) Job name which generates the artifact.
     *
     * @experimental
     */
    readonly job?: string;
    /**
     * (experimental) Relative path from repository root (`/`) to the pipeline configuration YAML file.
     *
     * @experimental
     */
    readonly file?: string;
    /**
     * (experimental) Path to another private project under the same GitLab instance, like `group/project` or `group/sub-group/project`.
     *
     * @experimental
     */
    readonly project?: string;
    /**
     * (experimental) Branch/Tag/Commit hash for the target project.
     *
     * @experimental
     */
    readonly ref?: string;
}
/**
 * You can mirror the pipeline status from the triggered pipeline to the source bridge job
 * by using strategy: depend
 * @see https://docs.gitlab.com/ee/ci/yaml/#triggerstrategy
 */
export declare enum Strategy {
    /**
     * (experimental) You can mirror the pipeline status from the triggered pipeline to the source bridge job by using strategy: depend.
     *
     * @see https://docs.gitlab.com/ee/ci/yaml/#triggerstrategy
     * @experimental
     */
    DEPEND = "depend"
}
/**
 * (experimental) Explains what the global variable is used for, what the acceptable values are.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#variables
 * @experimental
 */
export interface VariableConfig {
    /**
     * (experimental) Define a global variable that is prefilled when running a pipeline manually.
     *
     * Must be used with value.
     *
     * @experimental
     */
    readonly description?: string;
    /**
     * (experimental) The variable value.
     *
     * @experimental
     */
    readonly value?: string;
}
/**
 * (experimental) Used to control pipeline behavior.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#workflow
 * @experimental
 */
export interface Workflow {
    /**
     * (experimental) Used to control whether or not a whole pipeline is created.
     *
     * @experimental
     */
    readonly rules?: WorkflowRule[];
}
/**
 * (experimental) Used to control whether or not a whole pipeline is created.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#workflowrules
 * @experimental
 */
export interface WorkflowRule {
    /**
     * @experimental
     */
    readonly changes?: string[];
    /**
     * @experimental
     */
    readonly exists?: string[];
    /**
     * @experimental
     */
    readonly if?: string;
    /**
     * @experimental
     */
    readonly variables?: Record<string, number | string>;
    /**
     * @experimental
     */
    readonly when?: JobWhen;
}
/**
 * (experimental) Describes the conditions for when to run the job.
 *
 * Defaults to 'on_success'.
 * The value can only be 'always' or 'never' when used with workflow.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/#workflowrules
 * @experimental
 */
export declare enum WorkflowWhen {
    /**
     * @experimental
     */
    ALWAYS = "always",
    /**
     * @experimental
     */
    NEVER = "never"
}
