import { Component } from "../component";
import { Project } from "../project";
import { YamlFile } from "../yaml";
import { Artifacts, Cache, Default, Image, Include, Job, Retry, Service, VariableConfig, Workflow } from "./configuration-model";
/**
 * (experimental) Options for `CiConfiguration`.
 *
 * @experimental
 */
export interface CiConfigurationOptions {
    /**
     * (experimental) Default settings for the CI Configuration.
     *
     * Jobs that do not define one or more of the listed keywords use the value defined in the default section.
     *
     * @experimental
     */
    readonly default?: Default;
    /**
     * (experimental) A special job used to upload static sites to Gitlab pages.
     *
     * Requires a `public/` directory
     * with `artifacts.path` pointing to it.
     *
     * @experimental
     */
    readonly pages?: Job;
    /**
     * (experimental) Used to control pipeline behavior.
     *
     * @experimental
     */
    readonly workflow?: Workflow;
    /**
     * (experimental) Groups jobs into stages.
     *
     * All jobs in one stage must complete before next stage is
     * executed. If no stages are specified. Defaults to ['build', 'test', 'deploy'].
     *
     * @experimental
     */
    readonly stages?: string[];
    /**
     * (experimental) Global variables that are passed to jobs.
     *
     * If the job already has that variable defined, the job-level variable takes precedence.
     *
     * @experimental
     */
    readonly variables?: Record<string, any>;
    /**
     * (experimental) An initial set of jobs to add to the configuration.
     *
     * @experimental
     */
    readonly jobs?: Record<string, Job>;
}
/**
 * (experimental) CI for GitLab.
 *
 * A CI is a configurable automated process made up of one or more stages/jobs.
 *
 * @see https://docs.gitlab.com/ee/ci/yaml/
 * @experimental
 */
export declare class CiConfiguration extends Component {
    /**
     * (experimental) The project the configuration belongs to.
     *
     * @experimental
     */
    readonly project: Project;
    /**
     * (experimental) The name of the configuration.
     *
     * @experimental
     */
    readonly name: string;
    /**
     * (experimental) Path to CI file generated by the configuration.
     *
     * @experimental
     */
    readonly path: string;
    /**
     * (experimental) The workflow YAML file.
     *
     * @experimental
     */
    readonly file: YamlFile;
    /**
     * (experimental) Defines default scripts that should run *after* all jobs.
     *
     * Can be overriden by the job level `afterScript`.
     *
     * @experimental
     */
    readonly defaultAfterScript: string[];
    /**
     * (experimental) Default list of files and directories that should be attached to the job if it succeeds.
     *
     * Artifacts are sent to Gitlab where they can be downloaded.
     *
     * @experimental
     */
    readonly defaultArtifacts?: Artifacts;
    /**
     * (experimental) Defines default scripts that should run *before* all jobs.
     *
     * Can be overriden by the job level `afterScript`.
     *
     * @experimental
     */
    readonly defaultBeforeScript: string[];
    /**
     * (experimental) A default list of files and directories to cache between jobs.
     *
     * You can only use paths that are in the local working copy.
     *
     * @experimental
     */
    readonly defaultCache?: Cache;
    /**
     * (experimental) Specifies the default docker image to use globally for all jobs.
     *
     * @experimental
     */
    readonly defaultImage?: Image;
    /**
     * (experimental) The default behavior for whether a job should be canceled when a newer pipeline starts before the job completes (Default: false).
     *
     * @experimental
     */
    readonly defaultInterruptible?: boolean;
    /**
     * (experimental) How many times a job is retried if it fails.
     *
     * If not defined, defaults to 0 and jobs do not retry.
     *
     * @experimental
     */
    readonly defaultRetry?: Retry;
    /**
     * A default list of additional Docker images to run scripts in. The service image is linked to the image specified in the  image parameter.
     */
    private defaultServices;
    /**
     * (experimental) Used to select a specific runner from the list of all runners that are available for the project.
     *
     * @experimental
     */
    readonly defaultTags: string[];
    /**
     * (experimental) A default timeout job written in natural language (Ex.
     *
     * one hour, 3600 seconds, 60 minutes).
     *
     * @experimental
     */
    readonly defaultTimeout?: string;
    /**
     * Can be `Include` or `Include[]`. Each `Include` will be a string, or an
     * object with properties for the method if including external YAML file. The external
     * content will be fetched, included and evaluated along the `.gitlab-ci.yml`.
     */
    private include;
    /**
     * (experimental) A special job used to upload static sites to Gitlab pages.
     *
     * Requires a `public/` directory
     * with `artifacts.path` pointing to it.
     *
     * @experimental
     */
    readonly pages?: Job;
    /**
     * (experimental) Groups jobs into stages.
     *
     * All jobs in one stage must complete before next stage is
     * executed. Defaults to ['build', 'test', 'deploy'].
     *
     * @experimental
     */
    readonly stages: string[];
    /**
     * (experimental) Global variables that are passed to jobs.
     *
     * If the job already has that variable defined, the job-level variable takes precedence.
     *
     * @experimental
     */
    readonly variables: Record<string, number | VariableConfig | string>;
    /**
     * (experimental) Used to control pipeline behavior.
     *
     * @experimental
     */
    readonly workflow?: Workflow;
    /**
     * (experimental) The jobs in the CI configuration.
     *
     * @experimental
     */
    readonly jobs: Record<string, Job>;
    /**
     * @experimental
     */
    constructor(project: Project, name: string, options?: CiConfigurationOptions);
    /**
     * (experimental) Add additional yml/yaml files to the CI includes.
     *
     * @param includes The includes to add.
     * @experimental
     */
    addIncludes(...includes: Include[]): void;
    /**
     * Throw an error if the provided Include is invalid.
     * @see https://gitlab.com/gitlab-org/gitlab/-/blob/master/lib/gitlab/ci/config/external/mapper.rb
     * @param include the Include to validate.
     */
    private assertIsValidInclude;
    /**
     * Check if the equality of Includes.
     * @see https://gitlab.com/gitlab-org/gitlab/-/blob/master/lib/gitlab/ci/config/external/mapper.rb
     * @param x First include to compare.
     * @param y Second include to compare.
     * @returns Whether the includes are equal.
     */
    private areEqualIncludes;
    /**
     * (experimental) Add additional services.
     *
     * @param services The services to add.
     * @experimental
     */
    addServices(...services: Service[]): void;
    /**
     * (experimental) Add a globally defined variable to the CI configuration.
     *
     * @param variables The variables to add.
     * @experimental
     */
    addGlobalVariables(variables: Record<string, any>): void;
    /**
     * (experimental) Add stages to the CI configuration if not already present.
     *
     * @param stages stages to add.
     * @experimental
     */
    addStages(...stages: string[]): void;
    /**
     * (experimental) Add jobs and their stages to the CI configuration.
     *
     * @param jobs Jobs to add.
     * @experimental
     */
    addJobs(jobs: Record<string, Job>): void;
    private renderCI;
    private renderDefault;
}
